# Slixmpp: The Slick XMPP Library
# Copyright (C) 2020 Mathieu Pasquet <mathieui@mathieui.net>
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.plugins.base import register_plugin
from slixmpp.plugins.xep_0428.stanza import (
    Fallback, FallbackBody, FallbackSubject
)
from slixmpp.plugins.xep_0428.fallback import XEP_0428

register_plugin(XEP_0428)

__all__ = ['Fallback', 'FallbackBody', 'FallbackSubject', 'XEP_0428']
