from slixmpp.plugins.base import register_plugin
from slixmpp.plugins.xep_0122.stanza import FormValidation
from slixmpp.plugins.xep_0122.data_validation import XEP_0122


register_plugin(XEP_0122)

__all__ = ['FormValidation', 'XEP_0122']
